'''
Associated YT tutorial: https://youtu.be/TEWy9vZcxW4
'''

import numpy as np

np.random.seed(0)

X = [[1, 2, 3, 2.5],
     [2.0, 5.0, -1.0, 2.0],
     [-1.5, 2.7, 3.3, -0.8]]


class Layer_Dense:

    def __init__(self, n_inputs, n_neurons):
        self.weights = 0.10 * np.random.randn(n_inputs, n_neurons)
        self.biases = np.zeros((1, n_neurons))

    def forward(self, inputs):
        self.output = np.dot(inputs, self.weights) + self.biases


layer1 = Layer_Dense(4,5)
layer2 = Layer_Dense(5,2)

layer1.forward(X)
# print(layer1.output)
layer2.forward(layer1.output)
print(layer2.output)



import os
import sys
import shutil
import nnfs_video_tutorial_code


def print_usage():
    print('''
Neural Networks from Scratch in Python Tool.

Basic usage:
nnfs command [parameter1 [parameter2]]

Detailed usage:
nnfs info | code video_part [destination]

Commands:
  info    Prints information about the book
  code    Creates a file containing the code of given video part
          in given location. Location is optional, example:
          nnfs code 2 nnfs/p02.py
          will create a file p02.py in a nnfs folder containing
          the code of part 2 of video tutorial
    ''')


def info():
    print('''
  Neural Networks from Scratch in Python
  by Harrison Kinsley & Daniel Kukieła

  https://nnfs.io/

  ---

  This package contains the code and supplementary material
  as well as lesson code related to the book and video series.

    ''')


def code(video_part, path='/.'):

    final_path = path.lstrip("/").lstrip("\\")
    final_path = os.path.realpath(os.getcwd() + '/' + final_path)
    if path.rstrip('.').endswith('/') or path.rstrip('.').endswith('\\'):
        final_path += '/'

    try:
        int(video_part)
    except:
        print('Video part needs to be an integer')
        return

    code_folder = os.path.dirname(nnfs_video_tutorial_code.__file__)

    parts = [int(file.split('.')[0]) for file in os.listdir(code_folder) if os.path.isfile(code_folder + '/' + file) and not file.startswith('_')]
    if int(video_part) > max(parts):
        print('Video part does not exist')
        return

    try:
        os.makedirs(final_path if path.rstrip('.').endswith('/') or path.rstrip('.').endswith('\\') else os.path.dirname(final_path), exist_ok=True)
    except:
        print('Could not create one on more directories in a treefile ' + final_path + ' - permission issue?')

    if path.rstrip('.').endswith('/') or path.rstrip('.').endswith('\\'):
        final_path += 'p{:03d}.py'.format(int(video_part))

    try:
        shutil.copyfile(code_folder + '/{:03d}.py'.format(int(video_part)), final_path)
    except:
        print('Could not create file ' + final_path + 'permission issue?')

    print('Tutorial code for part ' + video_part + ' saved as ' + final_path)


def main():

    if len(sys.argv) == 1:
        print_usage()
        return

    command = sys.argv[1].strip()

    if command == 'info':
        info()
    elif command == 'code' and len(sys.argv) < 3:
        print_usage()
    elif command == 'code':
        code(*sys.argv[2:])



import numpy as np
import inspect


# Initializes NNFS
def init(dot_precision_workaround=True, default_dtype='float32', random_seed=0):

    # Numpy methods to be replaced for a workaround
    methods_to_enclose = [
        [np, 'zeros', False, 1],
        [np.random, 'randn', True, None],
        [np, 'eye', False, 1],
    ]

    # https://github.com/numpy/numpy/issues/15591
    # np.dot() is not consistent between machines
    # This workaround raises consistency
    # we make computations using float64, but return float32 data
    if dot_precision_workaround:
        orig_dot = np.dot
        def dot(*args, **kwargs):
            return orig_dot(*[a.astype('float64') for a in args], **kwargs).astype('float32')
        np.dot = dot
    else:
        methods_to_enclose.append([np, 'dot', True, None])

    # https://github.com/numpy/numpy/issues/6860
    # It is not possible to set default datatype with numpy
    # To make it able to enable and disable above workaround and set dtype
    # we'll enclose numpy method's with own one which will force different datatype
    if default_dtype:
        for method in methods_to_enclose:
            enclose(method, default_dtype)

    # Set seed to the given value (0 by default)
    if random_seed is not None:
        np.random.seed(random_seed)


# Encloses numpy method
def enclose(method, default_dtype):

    # Save a handler to original method
    method.append(getattr(*method[:2]))
    def enclosed_method(*args, **kwargs):

        # If flag is True - use .astype()
        if method[2]:
            return method[4](*args, **kwargs).astype(default_dtype)

        # Else pass dtype in kwargs
        else:
            if len(args) <= method[3] and 'dtype' not in kwargs:
                kwargs['dtype'] = default_dtype
            return method[4](*args, **kwargs)

    # Replace numpy method with enclosed one
    setattr(*method[:2], enclosed_method)



'''
Creates a simple layer of neurons, with 4 inputs.
Associated YT NNFS tutorial: https://www.youtube.com/watch?v=lGLto9Xd7bU
'''

inputs = [1.0, 2.0, 3.0, 2.5]

weights1 = [0.2, 0.8, -0.5, 1.0]
weights2 = [0.5, -0.91, 0.26, -0.5]
weights3 = [-0.26, -0.27, 0.17, 0.87]

bias1 = 2.0
bias2 = 3.0
bias3 = 0.5

output = [inputs[0]*weights1[0] + inputs[1]*weights1[1] + inputs[2]*weights1[2] + inputs[3]*weights1[3] + bias1,
          inputs[0]*weights2[0] + inputs[1]*weights2[1] + inputs[2]*weights2[2] + inputs[3]*weights2[3] + bias2,
          inputs[0]*weights3[0] + inputs[1]*weights3[1] + inputs[2]*weights3[2] + inputs[3]*weights3[3] + bias3]
print(output)



import numpy as np 
import nnfs
from nnfs.datasets import spiral_data  # See for code: https://gist.github.com/Sentdex/454cb20ec5acf0e76ee8ab8448e6266c

nnfs.init()

X, y = spiral_data(100, 3)   


class Layer_Dense:
    def __init__(self, n_inputs, n_neurons):
        self.weights = 0.10 * np.random.randn(n_inputs, n_neurons)
        self.biases = np.zeros((1, n_neurons))
    def forward(self, inputs):
        self.output = np.dot(inputs, self.weights) + self.biases


class Activation_ReLU:
    def forward(self, inputs):
        self.output = np.maximum(0, inputs)


layer1 = Layer_Dense(2,5)
activation1 = Activation_ReLU()

layer1.forward(X)

#print(layer1.output)
activation1.forward(layer1.output)
print(activation1.output)



'''
Doing dot product with a layer of neurons and multiple inputs
Associated YT NNFS tutorial: https://www.youtube.com/watch?v=tMrbN67U9d4
'''

import numpy as np 

inputs = [1.0, 2.0, 3.0, 2.5]
weights = [[0.2, 0.8, -0.5, 1.0],
           [0.5, -0.91, 0.26, -0.5],
           [-0.26, -0.27, 0.17, 0.87]]

biases = [2.0, 3.0, 0.5]

output = np.dot(weights, inputs) + biases
print(output)



import setuptools
import nnfs

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name="nnfs",
    version=nnfs.__version__,
    author="Harrison Kinsley",
    author_email="harrison@pythonprogramming.net",
    description="Package related to the Neural Networks from Scratch in Python book",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Sentdex/nnfs",
    project_urls={
        'Homepage': 'https://nnfs.io/',
        'Funding': 'https://nnfs.io/',
        'Documentation': 'https://nnfs.io/',
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Education",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Education",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    keywords='nnfs neural network networks from scratch in python',
    python_requires='>=3',
    install_requires=['numpy'],
    entry_points={
        "console_scripts": [
            "nnfs = nnfs.console.nnfs:main",
        ]
    }
)



'''
Creates a basic neuron with 3 inputs.

Associated YT NNFS tutorial: https://www.youtube.com/watch?v=Wo5dMEP_BbI
'''

inputs = [1.2, 5.1, 2.1]
weights = [3.1, 2.1, 8.7]
bias = 3.0

output = inputs[0]*weights[0] + inputs[1]*weights[1] + inputs[2]*weights[2] + bias
print(output)



import numpy as np


# Modified from:
# Copyright (c) 2015 Andrej Karpathy
# License: https://github.com/cs231n/cs231n.github.io/blob/master/LICENSE
# Source: https://cs231n.github.io/neural-networks-case-study/
def create_data(samples, classes):
    X = np.zeros((samples*classes, 2))
    y = np.zeros(samples*classes, dtype='uint8')
    for class_number in range(classes):
        ix = range(samples*class_number, samples*(class_number+1))
        X[ix] = np.c_[np.random.randn(samples)*.1 + (class_number)/3, np.random.randn(samples)*.1 + 0.5]
        y[ix] = class_number
    return X, y



import numpy as np


# Copyright (c) 2015 Andrej Karpathy
# License: https://github.com/cs231n/cs231n.github.io/blob/master/LICENSE
# Source: https://cs231n.github.io/neural-networks-case-study/
def create_data(samples, classes):
    X = np.zeros((samples*classes, 2))
    y = np.zeros(samples*classes, dtype='uint8')
    for class_number in range(classes):
        ix = range(samples*class_number, samples*(class_number+1))
        r = np.linspace(0.0, 1, samples)
        t = np.linspace(class_number*4, (class_number+1)*4, samples) + np.random.randn(samples)*0.2
        X[ix] = np.c_[r*np.sin(t*2.5), r*np.cos(t*2.5)]
        y[ix] = class_number
    return X, y
